//
//  CSV2StructurePlugIn.m
//  CSV2Structure
//
//  Created by Benot LAHOZ on 01/06/11.
//  Copyright (c) 2011 LAHOZ. All rights reserved.
//

/* It's highly recommended to use CGL macros instead of changing the current context for plug-ins that perform OpenGL rendering */
#import <OpenGL/CGLMacro.h>

#import "CSV2StructurePlugIn.h"
#import "parseCSV.h"

#define	kQCPlugIn_Name				@"CSV 2 Structure"
#define	kQCPlugIn_Description		@"CSV 2 Structure Beta \n\rL'ange Carasuelo 2011 - www.carasuelo.org \n\r Take a CSV file in input and output a structure of strings, numbers and colors. \n\rTo output a color, add the string \"RGBA\" at the end of your CSV header, then insert 4 floats separated by a blank in the fields. \n\rAdapted from Danilo Campos CSV Parser - http://blog.danilocampos.com/"

@implementation CSV2StructurePlugIn


@dynamic inputPath, outputStructure;


+ (NSDictionary*) attributes
{
	/*
	Return a dictionary of attributes describing the plug-in (QCPlugInAttributeNameKey, QCPlugInAttributeDescriptionKey...).
	*/
	
	return [NSDictionary dictionaryWithObjectsAndKeys:kQCPlugIn_Name, QCPlugInAttributeNameKey, kQCPlugIn_Description, QCPlugInAttributeDescriptionKey, nil];
}

+ (NSDictionary*) attributesForPropertyPortWithKey:(NSString*)key
{
	if ([key isEqualToString:@"inputPath"])
        return [NSDictionary dictionaryWithObjectsAndKeys:
				@"CSV Path", QCPortAttributeNameKey,
				nil];
	
	if ([key isEqualToString:@"outputStructure"])
		return [NSDictionary dictionaryWithObjectsAndKeys:
				@"Structure", QCPortAttributeNameKey,
				nil];
	
	return nil;
}

+ (QCPlugInExecutionMode) executionMode
{
	/*
	Return the execution mode of the plug-in: kQCPlugInExecutionModeProvider, kQCPlugInExecutionModeProcessor, or kQCPlugInExecutionModeConsumer.
	*/
	
	return kQCPlugInExecutionModeProcessor;
}

+ (QCPlugInTimeMode) timeMode
{
	/*
	Return the time dependency mode of the plug-in: kQCPlugInTimeModeNone, kQCPlugInTimeModeIdle or kQCPlugInTimeModeTimeBase.
	*/
	
	return kQCPlugInTimeModeNone;
}

- (id) init
{
	if(self = [super init]) {
		/*
		Allocate any permanent resource required by the plug-in.
		*/
	}
	
	return self;
}

- (void) finalize
{
	/*
	Release any non garbage collected resources created in -init.
	*/
	
	[super finalize];
}

- (void) dealloc
{
	/*
	Release any resources created in -init.
	*/
	
	[super dealloc];
}

- (id) testNumberWithString:(NSString*)string{
	
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	NSLocale *l_en = [[NSLocale alloc] initWithLocaleIdentifier: @"en_US"];
	NSNumberFormatter *f = [[NSNumberFormatter alloc] init];
	
	[f setNumberStyle:NSNumberFormatterDecimalStyle];
	
	
	[f setLocale: l_en];
	[f setAllowsFloats: YES];
	
	if([f numberFromString: string] != nil) {
		
		NSNumber *obj = [f numberFromString: string];
		return obj;
		
	} else {
		
		return string;
		
	}
	
	
	[l_en release];
	[f release];
	[pool release];
	
}


- (id) testColorWithString:(NSString*)string {
	
	NSScanner *scanner = [NSScanner scannerWithString:string];
	[scanner setCharactersToBeSkipped:[NSCharacterSet characterSetWithCharactersInString:@"\n, "]];
	
	float red, green, blue, alpha;
	CGFloat components[4];
	
	NSLog(@"Valid red scan: %@", [scanner scanFloat:&red]?@"YES":@"NO");
	NSLog(@"Valid green scan: %@", [scanner scanFloat:&green]?@"YES":@"NO");
	NSLog(@"Valid blue scan: %@", [scanner scanFloat:&blue]?@"YES":@"NO");
	NSLog(@"Valid alpha scan: %@", [scanner scanFloat:&alpha]?@"YES":@"NO");
	
	components[0] = (CGFloat)red;
	components[1] = (CGFloat)green;
	components[2] = (CGFloat)blue;
	components[3] = (CGFloat)alpha;
	
	if (scanner) {
	
		return CGColorCreate(colorSpace, components);
		
	} else {
	
		return string;
		
	}
	
}

@end

@implementation CSV2StructurePlugIn (Execution)

- (BOOL) startExecution:(id<QCPlugInContext>)context
{
	
	colorSpace = CGColorSpaceCreateWithName(kCGColorSpaceGenericRGB);
	
	return YES;
}

- (void) enableExecution:(id<QCPlugInContext>)context
{
	/*
	Called by Quartz Composer when the plug-in instance starts being used by Quartz Composer.
	*/
}

- (BOOL) execute:(id<QCPlugInContext>)context atTime:(NSTimeInterval)time withArguments:(NSDictionary*)arguments
{
	
	
	
	if([self didValueForInputKeyChange:@"inputPath"])
	{
		
		if([self.inputPath isAbsolutePath])
		{

			pathURL = self.inputPath;
			NSLog(@"URL absolue : %@", pathURL);
			
		}
		else
		{
			
			pathURL = [NSString pathWithComponents:[NSArray arrayWithObjects:[[[context compositionURL] path]stringByDeletingLastPathComponent], self.inputPath, nil]];
			[pathURL retain];
			
		}		

	}
	
	
	CSVParser *parser = [CSVParser new];
	

	if ([parser openFileWithPath:pathURL]) {
		
		NSLog(@"Path OK");
		
		NSMutableArray *csvContent = [parser parseFile];
		[parser closeFile];
		
		NSLog(@"Parse, csvContent : %@", csvContent);
		
		NSArray *keyArray = [csvContent objectAtIndex:0];
		
		NSMutableArray *plistOutputArray = [NSMutableArray array];
		
		NSInteger i = 0;
		
		for (NSArray *array in csvContent)
		{
			
			NSLog(@"Dans la boucle array");
			
			NSMutableDictionary *dictionary = [NSMutableDictionary dictionary];
			
			NSInteger keyNumber = 0;
			

			
			for (NSString *string in array)
			{
				
				NSLog(@"Dans la boucle string, keyNumber = %i", keyNumber);
				
				
				// CONVERTIT EN UTF16 =>>>>>>    A FAIRE DANS LE PARSER CSV !!!!!
				
				/*
				 NSData *data = [st dataUsingEncoding:NSUTF16StringEncoding allowLossyConversion:YES];
				 NSString *string = [[NSString alloc] initWithData:data encoding:NSUTF16StringEncoding];
				
				 NSLog(@"Original: %@ (length %d)", st, [st length]);	
				 NSLog(@"Converted: %@ (length %d)", string, [string length]);
				 */

				
				
				// NSLog(@"%@", [keyArray objectAtIndex:keyNumber]);
				
				if ([[keyArray objectAtIndex:keyNumber] hasSuffix:@"RGBA"]) {
				
					[dictionary setObject:[self testColorWithString:string] forKey:[keyArray objectAtIndex:keyNumber]];
				
				} else {
				
					[dictionary setObject:[self testNumberWithString:string] forKey:[keyArray objectAtIndex:keyNumber]];
										
				}
				
				keyNumber++;

				
			}
			
			if (i > 0)
			{
				[plistOutputArray addObject:dictionary];
			}
			
			i++;
			
		}
		
		//		NSLog(@"Write to URL %@",url);
		
	self.outputStructure = plistOutputArray;
	
	}
	
	
	
	
	return YES;
}

- (void) disableExecution:(id<QCPlugInContext>)context
{
	/*
	Called by Quartz Composer when the plug-in instance stops being used by Quartz Composer.
	*/
}

- (void) stopExecution:(id<QCPlugInContext>)context
{
	
	CGColorSpaceRelease(colorSpace);
	
}

@end
